// Dart imports:
import 'dart:async';

// Flutter imports:
import 'package:flutter/material.dart';

// Project imports:
import 'package:zego_uikit/src/components/message/in_room_message_view.dart';
import 'package:zego_uikit/src/components/message/live_commenting/in_room_live_commenting_view_item.dart';
import 'package:zego_uikit/zego_uikit.dart';

class ZegoInRoomNotificationView extends StatefulWidget {
  final int maxCount;
  final int itemMaxLine;
  final int itemDisappearTime; // ms
  final List<ZegoInRoomMessageType> dataSources; // data source of the view
  final MessageViewItemBuilder? itemBuilder;
  final bool isIMEnabled;

  const ZegoInRoomNotificationView({
    Key? key,
    this.maxCount = 3,
    this.itemMaxLine = 3,
    this.itemDisappearTime = 3000,
    this.dataSources = const [
      ZegoInRoomMessageType.userJoin,
      ZegoInRoomMessageType.userLeave,
      ZegoInRoomMessageType.remoteMessage
    ],
    this.itemBuilder,
    this.isIMEnabled = true,
  }) : super(key: key);

  @override
  State<ZegoInRoomNotificationView> createState() =>
      _ZegoInRoomNotificationViewState();
}

class _ZegoInRoomNotificationViewState
    extends State<ZegoInRoomNotificationView> {
  List<ZegoInRoomMessage> messages = []; //  chat/enter/leave messages,
  var messageListStream = StreamController<List<ZegoInRoomMessage>>.broadcast();
  List<StreamSubscription<dynamic>> streamSubscriptions = [];
  Timer? clearTimeoutTimer;

  @override
  void initState() {
    super.initState();

    clearTimeoutTimer =
        Timer.periodic(const Duration(seconds: 1), clearTimeoutMessage);

    if (widget.dataSources.contains(ZegoInRoomMessageType.remoteMessage) ||
        widget.dataSources.contains(ZegoInRoomMessageType.localMessage)) {
      streamSubscriptions
          .add(ZegoUIKit().getInRoomMessageStream().listen(onInRoomMessage));
    }
    if (widget.dataSources.contains(ZegoInRoomMessageType.userJoin)) {
      streamSubscriptions
          .add(ZegoUIKit().getUserJoinStream().listen(onUserJoin));
    }
    if (widget.dataSources.contains(ZegoInRoomMessageType.userLeave)) {
      streamSubscriptions
          .add(ZegoUIKit().getUserLeaveStream().listen(onUserLeave));
    }
  }

  @override
  void dispose() {
    super.dispose();

    clearTimeoutTimer?.cancel();
    for (var streamSubscription in streamSubscriptions) {
      streamSubscription.cancel();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ZegoInRoomMessageView(
      stream: messageListStream.stream,
      itemBuilder: widget.itemBuilder ??
          (BuildContext context, ZegoInRoomMessage message) {
            return ZegoInRoomLiveCommentingViewItem(
              maxLines: 3,
              // todo  how to show host?
              // prefix: message.user.id ==
              //         ZegoUIKit().getLocalUser().id
              //     ? 'Me'
              //     : null,
              user: message.user,
              message: message.message,
            );
          },
    );
  }

  void onUserJoin(List<ZegoUIKitUser> users) {
    for (var user in users) {
      if (user.id == ZegoUIKit().getLocalUser().id) {
        continue;
      }

      var message = ZegoInRoomMessage(
        user: user,
        message: "entered the room.",
        timestamp: DateTime.now().millisecondsSinceEpoch,
        messageID: 0,
        errorCode: 0,
      );
      messages.add(message);
    }
    broadcastStream();
  }

  void onUserLeave(List<ZegoUIKitUser> users) {
    for (var user in users) {
      if (user.id == ZegoUIKit().getLocalUser().id) {
        continue;
      }

      var message = ZegoInRoomMessage(
        user: user,
        message: "left the room.",
        timestamp: DateTime.now().millisecondsSinceEpoch,
        messageID: 0,
        errorCode: 0,
      );
      messages.add(message);
    }
    broadcastStream();
  }

  void onInRoomMessage(ZegoInRoomMessage message) {
    if (!widget.isIMEnabled) {
      return;
    }

    if (widget.dataSources.contains(ZegoInRoomMessageType.remoteMessage)) {
      if (message.user.id != ZegoUIKit().getLocalUser().id) {
        messages.add(message);
      }
    }

    if (widget.dataSources.contains(ZegoInRoomMessageType.localMessage)) {
      if (message.user.id == ZegoUIKit().getLocalUser().id) {
        messages.add(message);
      }
    }

    broadcastStream();
  }

  void clearTimeoutMessage(Timer _) {
    if (messages.isEmpty) {
      return;
    }

    messages.removeWhere((message) {
      var isTimeoutMessage = DateTime.now()
              .difference(
                  DateTime.fromMillisecondsSinceEpoch(message.timestamp))
              .inMilliseconds >
          widget.itemDisappearTime;
      return isTimeoutMessage;
    });

    broadcastStream();
  }

  void broadcastStream() {
    if (messages.length > widget.maxCount) {
      messages.removeRange(0, messages.length - widget.maxCount);
    }

    messageListStream.add(messages.take(widget.maxCount).toList());
  }
}
